function rfc = MissRatio_RegRFtrain(Xdata,param)
% rfc = MissRatio_RegRFtrain(Xdata,param)
% 
% train forest with missing data, adapting the 
% scheme of Regression Trees
%
% In particular, when a missing value is encountered
% there are 3 options (default: param.misstrain = 2) 
% 1. propagate points with NaN in both directions
% 2. propagate points with NaN in one random direction
% 3. stop points with NaN
% 
% Miss RatioRF Random Forest library
% (c) Manuele Bicego 2025
% 
% Xdata: [nxd]: n objects in a d-dimensional space
%        Nan indicates the presence of a missing value
%

global idn
[n,d] = size(Xdata);

if param.maxDepth == 0
    % automatic depth
    param.maxDepth = ceil(log2(n));
end
if param.sampling<=1
    % fraction
    psi = round(n*param.sampling);
else
    % number
    psi = param.sampling;
    if psi> n
        psi = n;
    end
end
i = 1;
if param.verbose, fprintf('Training \n'); end
while (i<=param.ntrees)
    % step 1: random sampling of a subset
    aa = randperm(n);
    bt = aa(1:psi);
    btx = Xdata(bt,:);
    % featfraction: select only a subset for training
    nf = max(1,round(d*param.featfraction));
    fss = randperm(d);
    fss = fss(1:nf);
    param.allf = fss;
    %btx = btx(:,fss);
    % step 2: train tree
    if (mod(i,10) == 0) || (i == 1) || (i == param.ntrees), if param.verbose, fprintf('Tree %d/%d\n',i,param.ntrees); end; end
    idn = 1;
    % remove objects for which we have all NaN in the selected features
    mi = isnan(btx(:,fss));
    btx(mean(mi,2)==1,:)=[]; % remove rows with all NaN
    % train tree
    t1 = MissRatio_regtreetrain(btx,param,1);
    rfc.trees{i} = t1;
    rfc.dep(i) = t1.maxdepth;
    i = i+1;
    
end
rfc.param = param;
