function [DD1,DD2,CI] = MissRatio_getRFDist_v2(Xdata,rfc,param,CI)
% [DD1,DD2,CI] = MissRatio_getRFDist(Xdata,rfc,param,CI)
% compute RatioRF distance from missing values
%
% Miss RatioRF Random Forest library
% (c) Manuele Bicego 2025
%
% Xdata: [nxd]: n objects in a d-dimensional space
%        Nan indicates the presence of a missing value
% rfc is the trained forest
% 
% DD1,DD2 are the two MissRatioRF variants (equations 9-10 of the paper)


if nargin < 4
    CI = MissRatio_getRFPathInfo(Xdata,rfc,param);
end
nT = length(CI);
n = size(Xdata,1);
% RatioRF
DD = zeros(n);
DDnum = zeros(n);
DDden = zeros(n);
for t = 1:nT
    if (mod(t,10) == 0) || (t == 1) || (t == param.ntrees), if param.verbose, fprintf('Tree %d/%d\n',t,param.ntrees); end; end
    xDD3 = zeros(n);
    xDD3num = zeros(n);
    xDD3den = zeros(n);
    beta = CI(t).beta;
    b = CI(t).b;
    % removing leaves
    d = (CI(t).feat == 0);
    beta(:,d) = [];
    b(:,d) = [];
    bnan = (b==2);
    % li = CI(t).leafid;
    for i = 1:n
        betai = beta(i,:);
        bi = b(i,:);
        allbeta = repmat(betai,n,1) | beta;
        allbi = (repmat(bi,n,1) == b);
        % removing nan answers
        bnanans = (bnan | repmat(bnan(i,:),n,1));
        allbi2 = allbi.*(1-bnanans);
        allbeta2 = allbeta.*(1-bnanans);
        % keeping only allbeta
        allbi3 = allbi2.*allbeta2;
        num = sum(allbi3,2);
        den = sum(allbeta2,2);
        xDD3(i,:) = num./(den + (den == 0));
        xDD3num(i,:) = num;
        xDD3den(i,:) = den;
    end
%     for i = 1:n
%         betai = beta(i,:);
%         bi = b(i,:);
%         binan = bnan(i,:);
%         for j = 1:n
%             betaj = beta(j,:);
%             bj = b(j,:);
%             bjnan = bnan(j,:);
%             
%             bin = 1-(binan | bjnan);
%             yallbi = (bi == bj);
%             yallbi2 = yallbi.*bin;
%             
%             allbeta = betai|betaj;
%             allbeta2 = allbeta.*bin;
%             
%             yallbi3 = yallbi2.*allbeta2;
%             
%             yDD(i,j) = sum(yallbi3,2)./sum(allbeta2,2);
%             
%         end
%     end
    
    
    % Note 22-3-22: I changed the way we compute the distance
    % I don't think we still need the correction
    %
    % the correction (li ~= li') considers mismatch between edges
    % (theory) and nodes (implementation)
    % is it still needed? should we put it where?
    
    % June 22
    % Correction: if one element is made of all Nan on the features of the
    % trees, its self similarity is zero --> correct 
    % by placing 1 on the diagonal
    rra = 1:(n+1):n*n;
    xDD3(rra) = 1;
    
    DD = DD+xDD3;
    DDnum = DDnum + xDD3num;
    DDden = DDden + xDD3den;
end
% Return both variants
%if param.missdist == 1
    DD1 = DD./nT;
%else
    DD2 = DDnum./DDden;
%end
DD1 = sqrt(1-DD1);
DD2 = sqrt(1-DD2);



