function info = MissRatio_getTreePathInfo(Xdata,t,param)
% info = MissRatio_getTreePathInfo(Xdata,t,param)
% 
% extract info from paths for distance computation (routine for a single tree)
%
% Miss RatioRF Random Forest library
% (c) Manuele Bicego 2025
% 
% Xdata: [nxd]: n objects in a d-dimensional space
%        Nan indicates the presence of a missing value
% rfc is the trained forest
%
% info.b [n X Nnodes]; % ij -> result of test of node j on object i (1 true, 0, false) 
% info.beta [n X Nnodes]; %ij -> true if node j is in the path of object i
% info.theta [1 X Nnodes]; % j -> threshold of test node j
% info.feat [1 X Nnodes]; % j -> feature on which make the test
% info.lev  [1 X Nnodes]; % j -> level of node j
% info.npathTest [n X 1]; % i -> n of tests for object i
%                           (in most of the cases just the depth)

n=size(Xdata,1);

Nn = t.maxidn; % number of nodes
b = zeros(n,Nn); % ij -> result of test of node j on object i (2 Nan, 1 true, 0 false) 
beta = false(n,Nn); %ij -> true if node j is in the path of object i
theta = zeros(1,Nn); % j -> threshold of test node j
feat = zeros(1,Nn); % j -> feature on which make the test
lev = zeros(1,Nn); % j -> level of node j
%npathTest = zeros(n,1); % i -> n of tests for object i
%                           (in most of the cases just the depth)

% scanning tree
tovisit{1} = t;
neltr = zeros(1,Nn);
while (~isempty(tovisit))
    v = tovisit{1};
    id = v.idn;
    lev(id) = v.liv;
    neltr(id) = v.sizet;
     if ~isempty(v.l)
         theta(id) = v.bestt;
         feat(id) = v.bestf;
         tovisit{end+1} = v.l; 
         tovisit{end+1} =v.r;
     end
     tovisit(1) = [];
end

for i = 1:n
    xi = Xdata(i,:);
    xinan = isnan(xi(feat(feat~=0)));
    % b contains 2 for NaN
    b(i,(feat~=0)) = 2*xinan;
    % result of tests
    cc = xi(feat(feat~=0)) >= theta(feat~=0);
    % write them only on not nan
    b(i,(feat~=0)) = b(i,(feat~=0)) + cc.*(1-xinan);
    % path
    v = t;
    tovisit= cell(1,Nn);
    nd = 1;
    tovisit{nd} = v;
    while (~isempty(tovisit{1}))
        v = tovisit{1};
        beta(i,v.idn) = 1;
        %npathTest(i) = npathTest(i)+1;
        if ~isempty(v.l) % leaf node
            if b(i,v.idn) == 2
                if param.misstest==1
                    % 1. Single path: one random path
                    nd = nd+1;
                    if randi(2) == 1
                        tovisit{nd} = v.l; 
                    else
                        tovisit{nd} = v.r; 
                    end
                else
                    % 2. Multi path: keep both directions
                    nd = nd+1;
                    tovisit{nd} = v.l; 
                    nd = nd+1;
                    tovisit{nd} = v.r;
                end
            else
                if (xi(v.bestf)<v.bestt)
                    nd = nd+1;
                    tovisit{nd} = v.l; 
                else
                    nd = nd+1;
                    tovisit{nd} = v.r;
                end
            end
        end
        tovisit(1) = [];
        nd = nd-1;
        if isempty(tovisit)
            break;
        end
    end
end
% leaf id: max(find(beta(1,:))
info.b = b;
info.beta = beta;
info.theta = theta;
info.feat = feat;
info.lev = lev;
%info.npathTest = npathTest;
